/******************************************************************************************
 *
 * CHECK_NTSTAT.C
 *
 * Program: NT stats server plugin for NetSaint
 * License: GPL
 * Copyright (c) 2000 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified by Ethan: 02-12-2000
 *
 * Command line: CHECK_NTSTAT <host_address> [-p port] [-v variable] [-wv warn_value] \
 *                            [-cv crit_value] [-to to_sec]
 *
 * Description:
 *
 * 
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************************/

#include "../common/config.h"
#include "../common/common.h"
#include "netutils.h"

#define CHECK_NONE				0
#define CHECK_PERCENT_USED_DISK			1
#define CHECK_USED_DISK				2
#define CHECK_PERCENT_USED_PHYS_MEM		3
#define CHECK_USED_PHYS_MEM			4
#define CHECK_PERCENT_USED_PAGED_MEM		5
#define CHECK_USED_PAGED_MEM			6
#define CHECK_CPU_LOAD_1			7
#define CHECK_CPU_LOAD_5			8
#define CHECK_CPU_LOAD_15			9

#define NTSTAT_PORT	5009

time_t start_time,end_time;
char server_address[MAX_HOST_ADDRESS_LENGTH];
int server_port=NTSTAT_PORT;
unsigned long warning_value=0L;
unsigned long critical_value=0L;
int check_warning_value=FALSE;
int check_critical_value=FALSE;
int var_to_check=CHECK_NONE;
int cmd_timeout=1;


/*int socket_timeout=DEFAULT_SOCKET_TIMEOUT;*/
extern int socket_timeout;

int check_all_cpus=TRUE;
int cpu_to_check=0;

char disk_name[MAX_INPUT_BUFFER];
char process_name[MAX_INPUT_BUFFER];

int process_arguments(int,char **);


int main(int argc, char **argv){
	int result;
	char send_buffer[MAX_INPUT_BUFFER];
	char recv_buffer[MAX_INPUT_BUFFER];
	char output_message[MAX_INPUT_BUFFER];
	char temp_buffer[MAX_INPUT_BUFFER];
	char *temp_ptr=NULL;
	int found_disk=FALSE;
	unsigned long percent_used_disk_space=0L;
	unsigned long total_disk_space=0L;
	unsigned long avail_disk_space=0L;
	unsigned long used_disk_space=0L;
	unsigned long percent_used_mem=0L;
	unsigned long total_mem=0L;
	unsigned long avail_mem=0L;
	unsigned long used_mem=0L;
	float cpu_load_1=0.0;
	float cpu_load_5=0.0;
	float cpu_load_15=0.0;

	result=process_arguments(argc,argv);

	if(result!=OK){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("NT stats plugin for NetSaint\n");
		printf("Copyright (c) 2000 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 02-12-2000\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [-p port] [-v variable] [-wv warn_value]\n",argv[0]);
		printf("       [-cv crit_value] [-to to_sec]\n");
		printf("\n");
		printf("Options:\n");
		printf(" [port]        = Optional port number to use - default is %d\n",server_port);
		printf(" [variable]    = Variable to check.  Valid variables include:\n");
		printf("		   PUPHYSMEM        = percent used physical memory\n");
		printf("                   UPHYSMEM         = used physical memory in MB\n");
		printf("                   PUPAGEMEM        = percent used paged memory\n");
		printf("                   UPAGEMEM         = used paged memory in MB\n");
		printf("                   PUDISK<disk>     = percent used disk space on <disk>\n");
		printf("                   UDISK<disk>      = used disk space on <disk> in MB\n");
		printf("                   LOAD1            = 1-minute average CPU load (all CPUs)\n");
		printf("                   LOAD5            = 5-minute average CPU load (all CPUs)\n");
		printf("                   LOAD15           = 15-minute average CPU load (all CPUs)\n");
		printf("                   CPUnLOAD1        = 1-minute average load on CPU #n (0-3)\n");
		printf("                   CPUnLOAD5        = 5-minute average load on CPU #n (0-3)\n");
		printf("                   CPUnLOAD15       = 15-minute average load on CPU #n (0-3)\n");
		printf(" [warn_value]  = Threshold for variable value necessary to result in\n");
		printf("                 a warning status\n");
		printf(" [crit_value]  = Threshold for variable value necessary to result in\n");
		printf("                 a critical status\n");
		printf(" [to_sec]      = Number of seconds before connection attempt times out.\n");
		printf("                 Default timeout is %d seconds\n",DEFAULT_SOCKET_TIMEOUT);
		printf("\n");
		printf("This plugin attempts to contact the NetSaint Stats Service running\n");
		printf("on the remote NT server to gather the requested system information.\n");
		printf("\n");
		printf("Notes:\n");
		printf("- For disk space queries the <disk> argument can be either a physical\n");
		printf("  drive (ie. 'C:', 'D:'), a directory name (with drive letter) or a\n");
		printf("  share.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* initialize alarm signal handling */
	signal(SIGALRM,socket_timeout_alarm_handler);

	/* set socket timeout */
	alarm(socket_timeout);
	
	result=STATE_OK;


	if(var_to_check==CHECK_PERCENT_USED_PHYS_MEM || var_to_check==CHECK_USED_PHYS_MEM || var_to_check==CHECK_PERCENT_USED_PAGED_MEM || var_to_check==CHECK_USED_PAGED_MEM){
	  
		if(var_to_check==CHECK_PERCENT_USED_PHYS_MEM || var_to_check==CHECK_USED_PHYS_MEM)
			sprintf(send_buffer,"TPHYSMEM APHYSMEM\r\n");
		else
			sprintf(send_buffer,"TPAGEMEM APAGEMEM\r\n");

		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;

		temp_ptr=(char *)strtok(recv_buffer," ");
		if(temp_ptr==NULL || !strcmp(temp_ptr,"-1")){
			sprintf(output_message,"Error getting total memory");
			result=STATE_CRITICAL;
		        }

		else{

			total_mem=((unsigned long)atol(temp_ptr))/1024;

			temp_ptr=(char *)strtok(NULL,"\r\n");
			if(temp_ptr==NULL || !strcmp(temp_ptr,"-1")){
				sprintf(output_message,"Error getting available memory");
				result=STATE_CRITICAL;
			        }

			else{

				avail_mem=((unsigned long)atol(temp_ptr))/1024;
				used_mem=total_mem-avail_mem;
				percent_used_mem=(used_mem*100)/total_mem;
		    
				if(var_to_check==CHECK_PERCENT_USED_PHYS_MEM || CHECK_PERCENT_USED_PAGED_MEM){

					if(check_critical_value==TRUE && (percent_used_mem >= critical_value))
						result=STATE_CRITICAL;
					else if(check_warning_value==TRUE && (percent_used_mem >= warning_value))
						result=STATE_WARNING;
				        }
				else{

					if(check_critical_value==TRUE && (used_mem >= (critical_value*1024)))
						result=STATE_CRITICAL;
					else if(check_warning_value==TRUE && (used_mem >= (warning_value*1024)))
						result=STATE_WARNING;
				        }

				sprintf(output_message,"%s memory %s - %2.1f MB (%lu%%) of %2.1f MB used",(var_to_check==CHECK_PERCENT_USED_PHYS_MEM || var_to_check==CHECK_USED_PHYS_MEM)?"Physical":"Paged",(result==STATE_OK)?"ok":"problem",(float)((float)used_mem/1024.0),percent_used_mem,(float)((float)total_mem/1024.0));
			        }
		        }
	        }

	else if(var_to_check==CHECK_PERCENT_USED_DISK || var_to_check==CHECK_USED_DISK){

		sprintf(send_buffer,"TDISK%s ADISK%s\r\n",disk_name,disk_name);
		
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;

		temp_ptr=(char *)strtok(recv_buffer," ");
		if(temp_ptr==NULL || !strcmp(temp_ptr,"-1")){
			sprintf(output_message,"Error getting total disk space");
			result=STATE_CRITICAL;
		        }

		else{
			total_disk_space=((unsigned long)atol(temp_ptr))/1024;

			temp_ptr=(char *)strtok(NULL,"\r\n");
			if(temp_ptr==NULL || !strcmp(temp_ptr,"-1")){
				sprintf(output_message,"Error getting available disk space");
				result=STATE_CRITICAL;
			        }

			else{

				avail_disk_space=((unsigned long)atol(temp_ptr))/1024;
				used_disk_space=total_disk_space-avail_disk_space;
				percent_used_disk_space=(used_disk_space*100)/total_disk_space;

				if(var_to_check==CHECK_PERCENT_USED_DISK){

					if(check_critical_value==TRUE && (percent_used_disk_space >= critical_value))
						result=STATE_CRITICAL;
					else if(check_warning_value==TRUE && (percent_used_disk_space >= warning_value))
						result=STATE_WARNING;
				        }
				else{

					if(check_critical_value==TRUE && (used_disk_space >= critical_value))
						result=STATE_CRITICAL;
					else if(check_warning_value==TRUE && (used_disk_space >= warning_value))
						result=STATE_WARNING;
				        }

				sprintf(output_message,"Disk space %s - %lu MB (%lu%%) of %lu MB used",(result==STATE_OK)?"ok":"problem",used_disk_space,percent_used_disk_space,total_disk_space);
		                }
		        }
	        }

	else if(var_to_check==CHECK_CPU_LOAD_1 || var_to_check==CHECK_CPU_LOAD_5 || var_to_check==CHECK_CPU_LOAD_15){

		if(check_all_cpus==TRUE)
			sprintf(send_buffer,"CPULOAD1 CPULOAD5 CPULOAD15\r\n");
		else
			sprintf(send_buffer,"CPU%dLOAD1 CPU%dLOAD5 CPU%dLOAD15\r\n",cpu_to_check,cpu_to_check,cpu_to_check);
		
		result=process_tcp_request(server_address,server_port,send_buffer,recv_buffer,sizeof(recv_buffer));
		if(result!=STATE_OK)
			return result;

		temp_ptr=(char *)strtok(recv_buffer," ");
		if(temp_ptr==NULL || !strcmp(temp_ptr,"-1")){
			sprintf(output_message,"Error getting cpu load");
			result=STATE_CRITICAL;
		        }

		else{
			cpu_load_1=(float)atof(temp_ptr);

			temp_ptr=strtok(NULL," ");
			if(temp_ptr==NULL || !strcmp(temp_ptr,"-1")){
				sprintf(output_message,"Error getting cpu load");
				result=STATE_CRITICAL;
			        }
			else{
				cpu_load_5=(float)atof(temp_ptr);

				temp_ptr=strtok(NULL,"\r\n");
				if(temp_ptr==NULL || !strcmp(temp_ptr,"-1")){
					sprintf(output_message,"Error getting cpu load");
					result=STATE_CRITICAL;
				        }
				else{
					cpu_load_15=(float)atof(temp_ptr);

					if(check_all_cpus==TRUE)
						sprintf(output_message,"Load %s: 1-min avg=%3.1f%%, 5-min avg=%3.1f%%, 15-min avg=%3.1f%%",(result==STATE_OK)?"ok":"problem",cpu_load_1,cpu_load_5,cpu_load_15);
					else
						sprintf(output_message,"CPU #%d Load %s: 1-min avg=%3.1f%%, 5-min avg=%3.1f%%, 15-min avg=%3.1f%%",cpu_to_check,(result==STATE_OK)?"ok":"problem",cpu_load_1,cpu_load_5,cpu_load_15);

				        }
			        }
		        }

	        }

	else{
		sprintf(output_message,"No valid check variable specified");
		result=STATE_CRITICAL;
	        }

	/* reset timeout */
	alarm(0);

	printf("%s\n",output_message);

	return result;
        }


/* process command-line arguments */
int process_arguments(int argc, char **argv){
	char buffer[MAX_INPUT_BUFFER];
	int x;

	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strncpy(server_address,argv[1],sizeof(server_address)-1);
	server_address[sizeof(server_address)-1]='\x0';

	/* process remaining arguments */
	for(x=3;x<=argc;x++){

		/* we got the port to use */
		if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the variable to monitor */
		else if(!strcmp(argv[x-1],"-v")){
			if(x<argc){
				
				if(!strcmp(argv[x],"PUPHYSMEM"))
					var_to_check=CHECK_PERCENT_USED_PHYS_MEM;
				else if(!strcmp(argv[x],"UPHYSMEM"))
					var_to_check=CHECK_USED_PHYS_MEM;
				else if(!strcmp(argv[x],"PUPAGEMEM"))
					var_to_check=CHECK_PERCENT_USED_PAGED_MEM;
				else if(!strcmp(argv[x],"UPAGEMEM"))
					var_to_check=CHECK_USED_PAGED_MEM;

				else if(strstr(argv[x],"PUDISK")==argv[x]){
					var_to_check=CHECK_PERCENT_USED_DISK;
					strncpy(disk_name,argv[x]+6,sizeof(disk_name)-1);
					disk_name[sizeof(disk_name)-1]='\x0';
				        }
				else if(strstr(argv[x],"UDISK")==argv[x]){
					var_to_check=CHECK_USED_DISK;
					strncpy(disk_name,argv[x]+5,sizeof(disk_name)-1);
					disk_name[sizeof(disk_name)-1]='\x0';
				        }

				else if(!strcmp(argv[x],"LOAD1"))
					var_to_check=CHECK_CPU_LOAD_1;
				else if(!strcmp(argv[x],"LOAD5"))
					var_to_check=CHECK_CPU_LOAD_5;
				else if(!strcmp(argv[x],"LOAD15"))
					var_to_check=CHECK_CPU_LOAD_15;
				else if(strstr(argv[x],"CPU")==argv[x]){
					if(strlen(argv[x])<9)
						return ERROR;
					cpu_to_check=atoi(argv[x]+3);
					check_all_cpus=FALSE;
					if(!strcmp(argv[x]+4,"LOAD1"))
						var_to_check=CHECK_CPU_LOAD_1;
					else if(!strcmp(argv[x]+4,"LOAD5"))
						var_to_check=CHECK_CPU_LOAD_5;
					else if(!strcmp(argv[x]+4,"LOAD15"))
						var_to_check=CHECK_CPU_LOAD_15;
					else
						return ERROR;
				        }
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the warning threshold for the variable value */
		else if(!strcmp(argv[x-1],"-wv")){
			if(x<argc){
				warning_value=(unsigned long)atol(argv[x]);
			        check_warning_value=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the critical threshold for the variable value */
		else if(!strcmp(argv[x-1],"-cv")){
			if(x<argc){
				critical_value=(unsigned long)atol(argv[x]);
				check_critical_value=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }

		/* we got the socket timeout */
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				socket_timeout=atoi(argv[x]);
				if(socket_timeout<=0)
					return ERROR;
			   
				x++;
			        }
			else
				return ERROR;
		        }

		/* else we got something else... */
		else
			return ERROR;
	        }

	return OK;
        }

